#include <Wire.h>
#include <MPU6050.h>
#include <SoftwareSerial.h>

SoftwareSerial mySerial(9, 8); // RX, TX for DFPlayer Mini
MPU6050 mpu;

const int16_t shakeThreshold = 1000; // Adjust this threshold based on your shaking intensity
const int16_t switchTrackThresholdZ = 500; // Adjust this threshold for switching to the next track
const int shakeDelay = 1000; // Delay to avoid continuous shakes
const int debounceDelay = 200; // Delay to debounce shaking events
const int startStopCooldown = 5000; // Cooldown period after starting playback
bool isPlaying = false;
bool isShaking = false;
bool canStartStop = true;
unsigned long lastShakeTime = 0; // Variable to store the last time a shake was detected
int16_t xAxisBaseline, yAxisBaseline, zAxisBaseline;

void sendCommand(uint8_t command, uint16_t argument = 0) {
  uint8_t highByte = (argument >> 8) & 0xFF;
  uint8_t lowByte = argument & 0xFF;

  mySerial.write((uint8_t)0x7E);
  mySerial.write((uint8_t)0xFF);
  mySerial.write((uint8_t)0x06);
  mySerial.write((uint8_t)command);
  mySerial.write((uint8_t)0x00);
  mySerial.write(highByte);
  mySerial.write(lowByte);
  mySerial.write((uint8_t)0xEF);

  Serial.print("Sent command: ");
  Serial.print(command, HEX);
  Serial.print(" Argument: ");
  Serial.println(argument);
}

void setup() {
  Serial.begin(9600);
  mySerial.begin(9600);

  // Initialize MPU6050
  Wire.begin();
  mpu.initialize();
  Serial.println("MPU6050 initialized.");

  // Initialize DFPlayer Mini
  sendCommand(0x02); // DFPLAYER_EQ_NORMAL
  sendCommand(0x06, 20); // DFPLAYER_SET_VOLUME, adjust volume as needed
  Serial.println("DFPlayer Mini initialized.");

  // Determine home position and baseline
  delay(2000); // Allow time for the sensor to stabilize
  mpu.getAcceleration(&xAxisBaseline, &yAxisBaseline, &zAxisBaseline);
  Serial.println("Home position and baseline determined.");
}

void loop() {
  int16_t ax, ay, az;
  mpu.getAcceleration(&ax, &ay, &az);

  Serial.print("Acceleration - X: ");
  Serial.print(ax);
  Serial.print(" Y: ");
  Serial.print(ay);
  Serial.print(" Z: ");
  Serial.println(az);

  // Calculate variations from the baseline
  int xVariation = abs(ax - xAxisBaseline);
  int yVariation = abs(ay - yAxisBaseline);
  int zVariation = abs(az - zAxisBaseline);

  // Print variations for debugging
  Serial.print("Variations - X: ");
  Serial.print(xVariation);
  Serial.print(" Y: ");
  Serial.print(yVariation);
  Serial.print(" Z: ");
  Serial.println(zVariation);

  // Check for a change in shaking state
  if ((xVariation > shakeThreshold || yVariation > shakeThreshold || zVariation > shakeThreshold) && !isShaking) {
    lastShakeTime = millis();
    isShaking = true;
  } else if (isShaking && millis() - lastShakeTime > shakeDelay) {
    isShaking = false;
    canStartStop = true; // Allow new start/stop events
  }

  // If shaking started
  if (isShaking && canStartStop) {
    if (!isPlaying) {
      isPlaying = true;
      sendCommand(0x0D); // DFPLAYER_PLAY
      Serial.println("Playback started.");
      lastShakeTime = millis(); // Reset timer for cooldown
    }
  }
  // If shaking stopped
  else {
    if (isPlaying && millis() - lastShakeTime > startStopCooldown) {
      isPlaying = false;
      sendCommand(0x16); // DFPLAYER_PAUSE
      Serial.println("Playback paused.");
      canStartStop = false; // Prevent new start/stop events for a cooldown period
      lastShakeTime = millis(); // Reset timer for cooldown
    }
  }

  // Check for sudden shake (acceleration change) to switch to the next track
  if (zVariation > switchTrackThresholdZ && millis() - lastShakeTime > shakeDelay) {
    lastShakeTime = millis();

    // Check orientation for vertical shake (Y-axis)
    if (ay > yAxisBaseline) {
      if (isPlaying) {
        sendCommand(0x01); // DFPLAYER_NEXT
        Serial.println("Switched to the next track.");
      }
    }
  }

  delay(100);
}
